import {
  setQuerybuilderNodes,
  useAppDispatch,
  useConfig,
  useQuerybuilderGraph,
  useQuerybuilderGraphology,
  useSchemaGraph,
  useSchemaGraphology,
} from '@graphpolaris/shared/lib/data-access/store';
import ReactFlow, {
  ReactFlowProvider,
  Background,
  Node,
  isNode,
  ReactFlowInstance,
  Controls,
  ControlButton,
  NodeChange,
  ConnectionMode,
  NodeMouseHandler,
  OnConnectStartParams,
  useReactFlow,
  Edge,
  Connection,
  HandleType,
  XYPosition,
  Position,
  OnEdgesChange,
  NodePositionChange,
} from 'reactflow';
import styles from './querybuilder.module.scss';
import CachedIcon from '@mui/icons-material/Cached';

import React, { ReactComponentElement, useMemo, useRef, useEffect, useCallback, useState, DragEventHandler } from 'react';
import { AttributePill, ConnectionDragLine, ConnectionLine, EntityFlowElement, RelationPill } from '../pills';
import { dragPillStarted, dragPillStopped, movePillTo } from '../pills/dragging/dragPill';
import { Settings as SettingsIcon, Delete as DeleteIcon, ImportExport as ExportIcon } from '@mui/icons-material';
import { clearQB } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { RelationPosToFromEntityPos, RelationPosToToEntityPos } from '@graphpolaris/shared/lib/querybuilder/model/graphology/utils';
import { useDispatch } from 'react-redux';
import {
  Box,
  Card,
  CardContent,
  Dialog,
  DialogTitle,
  Grid,
  List,
  ListItem,
  ListItemButton,
  ListItemText,
  Paper,
  PaperProps,
  Tab,
  Tabs,
  Typography,
} from '@mui/material';
import {
  AllLogicDescriptions,
  AllLogicMap,
  Handles,
  MathFilters,
  StringFilters,
  MathFunctions,
  NodeAttribute,
  QueryElementTypes,
  QueryGraphNodes,
  StringFunctions,
  createReactFlowElements,
  toHandleData,
  QueryGraphEdgeHandle,
} from '../model';
import Draggable from 'react-draggable';
import { GeneralDescription, InputNodeType } from '../model/logic/general';
import LogicPill from '../pills/customFlowPills/logicpill/logicpill';
import { current } from '@reduxjs/toolkit';
import { SchemaAttributeTypes } from '../../schema';
import { MathAggregations } from '../model/logic/mathAggregations';

const nodeTypes = {
  entity: EntityFlowElement,
  relation: RelationPill,
  logic: LogicPill,
  attribute: AttributePill,
};
const edgeTypes = {
  connection: ConnectionLine,
  attribute_connection: ConnectionLine,
};

/**
 * This is the main querybuilder component. It is responsible for holding all pills and fire off the visual part of the querybuilder panel logic
 */
export const QueryBuilderInner = (props: QueryBuilderProps) => {
  const [openPopup, setOpenPopup] = useState(false);
  const reactFlowWrapper = useRef<HTMLDivElement>(null);

  const schema = useSchemaGraphology();
  const graphologyGraph = useQuerybuilderGraphology();
  const graph = useQuerybuilderGraph();
  const config = useConfig();
  const dispatch = useDispatch();
  const isDraggingPill = useRef(false);
  const connectingNodeId = useRef<{
    params: OnConnectStartParams;
    position: XYPosition;
    node: QueryGraphNodes;
    attribute: NodeAttribute;
  } | null>(null);
  const reactFlow = useReactFlow();
  const isEdgeUpdating = useRef(false);
  const isOnConnect = useRef(false);

  const onInit = (reactFlowInstance: ReactFlowInstance) => {
    setTimeout(() => reactFlow.fitView(), 0);
  };

  const elements = useMemo(() => createReactFlowElements(graphologyGraph), [graph]);

  /**
   * Called when a node is dragged in querybuilder to allow for movement
   * @param event
   * @param node
   */
  const onNodeDrag = (event: React.MouseEvent<Element, MouseEvent>, node: Node<any>, nodes: Node[]) => {
    // console.log(nodes);
    // // Get the node in the elements list to get the previous location
    // const pNode = elements.nodes.find((e) => e?.id === node?.id);
    // if (!(pNode && isNode(pNode))) return;
    // // This is then used to calculate the delta position
    // const dx = node.position.x - pNode.position.x;
    // const dy = node.position.y - pNode.position.y;
    // // Check if we started dragging, if so, call the drag started usecase
    // if (!isDraggingPill.current) {
    //   dragPillStarted(node.id, graphologyGraph);
    //   isDraggingPill.current = true;
    // }
    // // Call the drag usecase
    // movePillTo(node.id, graphologyGraph, dx, dy, node.position);
    // // Dispatch the new graphology object, so reactflow will get rerendered
    // dispatch(setQuerybuilderNodes(graphologyGraph.export()));
  };

  /**
   * Called when a node is released from dragging in querybuilder
   * @param event
   * @param node
   **/
  const onNodeDragStop = (event: React.MouseEvent<Element, MouseEvent>, node: Node<any>) => {
    // isDraggingPill.current = false;
    //
    // // Call the drag pill stopped usecase
    // dragPillStopped(node.id, graphologyGraph);
    //
    // // Dispatch the new graphology object, so reactflow will get rerendered
    // dispatch(setQuerybuilderNodes(graphologyGraph.export()));
  };

  /**
   * Clears all nodes in the graph.
   */
  function clearAllNodes() {
    dispatch(clearQB());
  }

  /**
   * Clears all nodes in the graph.
   *  TODO: only works if the node is clicked and not moved (maybe use onSelectionChange)
   */
  function onNodesDelete(nodes: Node[]) {
    nodes.forEach((n) => {
      graphologyGraph.dropNode(n.id);
    });

    // Dispatch the new graphology object, so reactflow will get re-rendered
    dispatch(setQuerybuilderNodes(graphologyGraph.export()));
  }

  /**
   * TODO?
   */
  function onNodesChange(nodes: NodeChange[]) {
    // console.log(nodes);
    nodes.forEach((n) => {
      // console.log(graphologyGraph.findNode((gn) => gn === n?.id));
      if (n.type !== 'position') {
        // updated = true;
        // graphologyGraph.updateAttributes(n.id, n.data);
      } else {
        const node = n as NodePositionChange;
        // Get the node in the elements list to get the previous location
        const pNode = elements.nodes.find((e) => e?.id === node?.id);
        if (!(pNode && isNode(pNode)) || !node?.position) return;
        // This is then used to calculate the delta position
        const dx = node.position.x - pNode.position.x;
        const dy = node.position.y - pNode.position.y;
        // Check if we started dragging, if so, call the drag started usecase
        if (!isDraggingPill.current) {
          dragPillStarted(node.id, graphologyGraph);
          isDraggingPill.current = true;
        }
        // Call the drag usecase
        movePillTo(node.id, graphologyGraph, dx, dy, node.position);
        // Dispatch the new graphology object, so reactflow will get rerendered
        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
      }
    });
  }

  const onDragOver = (event: React.DragEvent<HTMLDivElement>): void => {
    event.preventDefault();
    event.dataTransfer.dropEffect = 'move';
  };

  /**
   * The onDrop is called when the user drops an element from the schema onto the QueryBuilder.
   * In the onDrop query elements will be created based on the data stored in the drag event (datastrasfer).
   * @param event Drag event.
   */
  const onDrop = (event: React.DragEvent<HTMLDivElement>): void => {
    event.preventDefault();

    // The dropped element should be a valid reactflow element
    const data: string = event.dataTransfer.getData('application/reactflow');
    if (data.length == 0 || !reactFlow) return;

    const dragData = JSON.parse(data);
    console.log(reactFlowWrapper);

    const bounds = reactFlowWrapper?.current?.getBoundingClientRect() || { x: 0, y: 0 };
    const position = reactFlow.project({
      //TODO: this position should be centre of entity, rather than topleft
      x: event.clientX - bounds.x,
      y: event.clientY - bounds.y,
    });

    switch (dragData.type) {
      case QueryElementTypes.Entity:
        // console.log('entity drop', dragData, schema.getNodeAttribute(dragData.name, 'attributes'));
        graphologyGraph.addPill2Graphology(
          {
            type: QueryElementTypes.Entity,
            x: position.x,
            y: position.y,
            name: dragData.name,
          },
          schema.getNodeAttribute(dragData.name, 'attributes')
        );
        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
        break;
      // Creates a relation element and will also create the 2 related entities together with the connections
      case QueryElementTypes.Relation:
        console.log('relation drop', dragData, schema.export());

        const relation = graphologyGraph.addPill2Graphology(
          {
            type: QueryElementTypes.Relation,
            x: position.x,
            y: position.y,
            depth: { min: 0, max: 1 },
            name: dragData.collection,
            collection: dragData.collection,
          },
          schema.getEdgeAttribute(dragData.label, 'attributes')
        );
        // const leftEntity = graphologyGraph.addPill2Graphology(
        //   { type: QueryElementTypes.Entity, ...RelationPosToFromEntityPos(position), name: dragData.from },
        //   schema.getNodeAttribute(dragData.from, 'attributes')
        // );
        // const rightEntity = graphologyGraph.addPill2Graphology(
        //   { type: QueryElementTypes.Entity, ...RelationPosToToEntityPos(position), name: dragData.to },
        //   schema.getNodeAttribute(dragData.to, 'attributes')
        // );

        // graphologyGraph.addEdge2Graphology(leftEntity, relation);
        // graphologyGraph.addEdge2Graphology(relation, rightEntity);

        if (config.autoSendQueries) {
          // sendQuery();
        }

        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
        break;
      // Creates an attribute element with the correct dataType
      // case QueryElementTypes.Attribute:
      //   createNodeFromSchema(
      //     QueryElementTypes.Attribute,
      //     position,
      //     dragData.name,
      //     dragData.datatype
      //   );
      //   break;
      default:
        const logic = AllLogicMap[dragData.value.key];
        const firstLeftLogicInput = logic.inputs?.[0];
        if (!firstLeftLogicInput) return;

        // logicAttributes[0].handles = [connectingNodeId.current.handleId];
        const logicNode = graphologyGraph.addLogicPill2Graphology({
          name: dragData.value.name,
          type: QueryElementTypes.Logic,
          x: position.x,
          y: position.y,
          logic: logic,
        });

        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
    }
  };

  const onNodeMouseEnter = (event: React.MouseEvent, node: Node) => {
    console.log(node);
    console.log(schema.getNodeAttribute(node.type, 'attributes'));
  };

  const onNodeMouseLeave = (event: React.MouseEvent, node: Node) => {
    console.log(node);
  };

  const onConnect = useCallback(
    (connection: Connection) => {
      if (!isEdgeUpdating.current) {
        isOnConnect.current = true;
        if (!connection.sourceHandle || !connection.targetHandle) throw new Error('Connection has no source or target');
        console.log('onConnect', connection);

        if (!graphologyGraph.hasEdge(connection.source, connection.target)) {
          graphologyGraph.addEdge(connection.source, connection.target, {
            type: 'connection',
            sourceHandleData: toHandleData(connection.sourceHandle),
            targetHandleData: toHandleData(connection.targetHandle),
          });
          dispatch(setQuerybuilderNodes(graphologyGraph.export()));
        }
      }
    },
    [graph]
  );

  const onConnectStart = useCallback(
    (event: React.MouseEvent | React.TouchEvent, params: OnConnectStartParams) => {
      // console.log('onConnectStart', params);
      if (!params?.handleId) return;

      let node = graphologyGraph.getNodeAttributes(params.nodeId);
      const handleData = toHandleData(params.handleId);
      // console.log(attributeName, attributeType, node.attributes?.filter((a) => a.name === attributeName)?.[0]);

      connectingNodeId.current = {
        params,
        node,
        position: { x: 0, y: 0 },
        attribute: { handleData: handleData },
      };
    },
    [graph]
  );

  const onConnectEnd = useCallback(
    (event: any) => {
      const targetIsPane = event.target.classList.contains('react-flow__pane');
      if (isOnConnect.current) {
        isOnConnect.current = false;
        return;
      }

      if (targetIsPane && !isEdgeUpdating.current) {
        let clientX: number = 0;
        let clientY: number = 0;

        if ('touches' in event) clientX = event?.touches?.[0]?.clientX;
        else if ('clientX' in event) clientX = event?.clientX;
        if ('touches' in event) clientY = event?.touches?.[0]?.clientY;
        else if ('clientY' in event) clientY = event?.clientY;

        if (reactFlowWrapper.current) {
          const { top, left } = reactFlowWrapper.current.getBoundingClientRect();
          clientX -= left;
          clientY -= top;
        }
        const position = reactFlow.project({ x: clientX, y: clientY });
        if (connectingNodeId?.current) connectingNodeId.current.position = position;

        setOpenPopup(true);
      }
    },
    [reactFlow.project]
  );

  const onNewNodeFromPopup = (value: AllLogicDescriptions, type: InputNodeType) => {
    if (connectingNodeId.current === null || connectingNodeId.current?.params?.handleId == null) return;
    const params = connectingNodeId.current.params;
    const position = connectingNodeId.current.position;

    // console.log('onNewNodeFromPopup', value, type, params, position);

    const logic = AllLogicMap[value.key];
    const firstLeftLogicInput = logic.inputs?.[0];
    if (!firstLeftLogicInput) return;

    // logicAttributes[0].handles = [connectingNodeId.current.handleId];
    const logicNode = graphologyGraph.addLogicPill2Graphology({
      name: value.name,
      type: QueryElementTypes.Logic,
      x: position.x,
      y: position.y,
      logic: logic,
    });

    if (!logicNode?.id) throw new Error('Logic node has no id');
    if (!logicNode?.name) throw new Error('Logic node has no name');
    if (!params.handleId) throw new Error('Connection has no source or target');

    const sourceHandleData = toHandleData(params.handleId);
    graphologyGraph.addEdge2Graphology(
      graphologyGraph.getNodeAttributes(params.nodeId),
      graphologyGraph.getNodeAttributes(logicNode.id),
      { type: 'connection' },
      { sourceHandleName: sourceHandleData.attributeName, targetHandleName: firstLeftLogicInput.name }
    );

    dispatch(setQuerybuilderNodes(graphologyGraph.export()));
    setOpenPopup(false);
  };

  const onEdgeUpdateStart = useCallback(() => {
    isEdgeUpdating.current = true;
  }, []);

  const onEdgeUpdate = useCallback(
    (oldEdge: Edge, newConnection: Connection) => {
      if (isEdgeUpdating.current) {
        isEdgeUpdating.current = false;

        if (graphologyGraph.hasEdge(oldEdge.id)) {
          graphologyGraph.dropEdge(oldEdge.id);
        }

        if (!newConnection.sourceHandle || !newConnection.targetHandle) throw new Error('Connection has no source or target');
        graphologyGraph.addEdge(newConnection.source, newConnection.target, {
          type: 'connection',
          sourceHandleData: toHandleData(newConnection.sourceHandle),
          targetHandleData: toHandleData(newConnection.targetHandle),
        });
        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
      }
    },
    [graph]
  );

  const onEdgesChange = (params: OnEdgesChange) => {
    console.log('edges change', params);
  };

  const onEdgeUpdateEnd = useCallback(
    (event: MouseEvent | TouchEvent, edge: Edge, handleType: HandleType) => {
      if (isEdgeUpdating.current) {
        if (graphologyGraph.hasEdge(edge.id)) {
          graphologyGraph.dropEdge(edge.id);
        }
        dispatch(setQuerybuilderNodes(graphologyGraph.export()));
        // setEdges((eds) => eds.filter((e) => e.id !== edge.id));
      }
      isEdgeUpdating.current = false;
    },
    [graph]
  );

  const onNodeContextMenu = (event: React.MouseEvent, node: Node) => {
    event.preventDefault();
    // console.log('context menu', node);
    graphologyGraph.dropNode(node.id);
    dispatch(setQuerybuilderNodes(graphologyGraph.export()));
  };

  return (
    <div ref={reactFlowWrapper} className={styles.full}>
      <PopupMenu
        open={openPopup}
        type={connectingNodeId.current?.attribute.handleData.attributeType || 'int'}
        onClose={() => {
          setOpenPopup(false);
        }}
        onClick={onNewNodeFromPopup}
      />
      <ReactFlow
        edges={elements.edges}
        nodes={elements.nodes}
        snapGrid={[10, 10]}
        snapToGrid
        nodeTypes={nodeTypes}
        edgeTypes={edgeTypes}
        connectionLineComponent={ConnectionDragLine}
        // connectionMode={ConnectionMode.Loose}
        onInit={onInit}
        onNodesChange={onNodesChange}
        onNodeDrag={onNodeDrag}
        onNodeDragStop={onNodeDragStop}
        onDragOver={onDragOver}
        onConnect={onConnect}
        onConnectStart={onConnectStart}
        onConnectEnd={onConnectEnd}
        // onNodeMouseEnter={onNodeMouseEnter}
        // onNodeMouseLeave={onNodeMouseLeave}
        onEdgeUpdate={onEdgeUpdate}
        onEdgeUpdateStart={onEdgeUpdateStart}
        onEdgeUpdateEnd={onEdgeUpdateEnd}
        onDrop={onDrop}
        // onContextMenu={onContextMenu}
        onNodeContextMenu={onNodeContextMenu}
        // onNodesDelete={onNodesDelete}
        // onNodesChange={onNodesChange}
        deleteKeyCode="Backspace"
        className={styles.reactflow}
        attributionPosition="top-right"
      >
        <Background gap={10} size={0.7} />
        {elements && elements.edges.length == 0 && elements.nodes.length == 0 && (
          <Card variant="outlined" sx={{ minWidth: 275, marginTop: 3, marginRight: 10 }}>
            <CardContent>
              <Typography sx={{ fontSize: 20 }} color="text.secondary">
                Drag some node from the Schema panel
              </Typography>
            </CardContent>
          </Card>
        )}

        <Controls showZoom={false} showInteractive={false} className={styles.controls}>
          <ControlButton className={styles.buttons} title={'Remove all elements'} onClick={() => clearAllNodes()}>
            <DeleteIcon />
          </ControlButton>
          <ControlButton
            className={styles.buttons}
            title={'Export querybuilder'}
            onClick={(event) => {
              event.stopPropagation();
              // this.setState({
              //   ...this.state,
              //   exportMenuAnchor: event.currentTarget,
              // });
            }}
          >
            <ExportIcon />
          </ControlButton>
          <ControlButton
            className={styles.buttons}
            title={'Other settings'}
            onClick={(event) => {
              event.stopPropagation();
              // this.setState({
              //   ...this.state,
              //   settingsMenuAnchor: event.currentTarget,
              // });
            }}
          >
            <SettingsIcon />
          </ControlButton>
          <ControlButton
            className={styles.buttons}
            title={'Run Query'}
            onClick={(event) => {
              event.stopPropagation();
              if (props.onRunQuery) props.onRunQuery();
            }}
          >
            <CachedIcon />
          </ControlButton>
        </Controls>
      </ReactFlow>
    </div>
  );
};

// const math_functions = ['AVG', 'COUNT', 'MAX', 'MIN', 'SUM', 'ROUND', 'CEIL', 'FLOOR', '+', '-', '*', '/', '%', 'CUSTOM'];
// const string_functions = ['CONCAT', 'LOWER', 'UPPER', 'SUBSTRING', 'TRIM'];
// const math_filters = ['==', '!=', '>', '<', '>=', '<='];
// const string_filters = ['==', '!=', 'LIKE', 'NOT LIKE', 'IN', 'NOT IN'];

function PaperComponent(props: PaperProps) {
  return (
    <Draggable handle="#draggable-dialog-title" cancel={'[class*="MuiDialogContent-root"]'}>
      <Paper {...props} />
    </Draggable>
  );
}

interface TabPanelProps {
  children?: React.ReactNode;
  index: number;
  value: number;
}

function CustomTabPanel(props: TabPanelProps) {
  const { children, value, index, ...other } = props;

  return (
    <div
      role="tabpanel"
      hidden={value !== index}
      id={`simple-tabpanel-${index}`}
      aria-labelledby={`simple-tab-${index}`}
      {...other}
      style={{ maxHeight: '85%', overflow: 'auto' }}
    >
      {value === index && (
        <Box>
          <Typography>{children}</Typography>
        </Box>
      )}
    </div>
  );
}

function PopupMenu(props: {
  open: boolean;
  type: InputNodeType;
  onClose: () => void;
  onClick: (value: AllLogicDescriptions, type: InputNodeType) => void;
}) {
  const [value, setValue] = React.useState(props.type === 'string' ? 1 : 0);

  const handleChange = (event: React.SyntheticEvent, newValue: number) => {
    setValue(newValue);
  };

  const handleClose = () => {
    props.onClose();
  };

  const handleListItemClick = (value: AllLogicDescriptions, type: InputNodeType) => {
    props.onClick(value, type);
  };

  const generateList = (list: Record<string, AllLogicDescriptions>, type: InputNodeType) => (
    <List sx={{ pt: 0 }}>
      {Object.keys(list).map((f, i) => (
        <ListItemButton onClick={() => handleListItemClick(list[f], type)} key={f + type}>
          {/* <ListItemAvatar>
                <Avatar sx={{ bgcolor: blue[100], color: blue[600] }}>
                  <PersonIcon />
                </Avatar>
              </ListItemAvatar> */}
          <ListItemText primary={list[f].name} secondary={list[f].description} />
        </ListItemButton>
      ))}
    </List>
  );

  function a11yProps(index: number) {
    return {
      id: `simple-tab-${index}`,
      'aria-controls': `simple-tabpanel-${index}`,
    };
  }

  return (
    <Dialog onClose={handleClose} open={props.open} PaperComponent={PaperComponent}>
      <DialogTitle>Add New Node</DialogTitle>
      <Box>
        <Box sx={{ borderBottom: 1, borderColor: 'divider' }}>
          <Tabs value={value} onChange={handleChange} aria-label="basic tabs example">
            <Tab label="Aggregations" disabled={props.type === 'string'} {...a11yProps(0)} />
            <Tab label="Operations" {...a11yProps(1)} />
            <Tab label="Filters" {...a11yProps(2)} />
          </Tabs>
        </Box>
        <CustomTabPanel value={value} index={0}>
          {props.type === 'float' && generateList(MathAggregations, props.type)}
          {props.type === 'int' && generateList(MathAggregations, props.type)}
          {/* {props.type === 'string' && generateList(MathAggregations, props.type)} */}
        </CustomTabPanel>
        <CustomTabPanel value={value} index={1}>
          {props.type === 'float' && generateList(MathFunctions, props.type)}
          {props.type === 'int' && generateList(MathFunctions, props.type)}
          {props.type === 'string' && generateList(StringFunctions, props.type)}
        </CustomTabPanel>
        <CustomTabPanel value={value} index={2}>
          {props.type === 'float' && generateList(MathFilters, props.type)}
          {props.type === 'int' && generateList(MathFilters, props.type)}
          {props.type === 'string' && generateList(StringFilters, props.type)}
        </CustomTabPanel>
      </Box>
    </Dialog>
  );
}

export const QueryBuilderPills = () => {
  const onDragStart = (event: React.DragEvent, value: AllLogicDescriptions, nodeType: InputNodeType) => {
    console.log('drag start', nodeType);

    event.dataTransfer.setData('application/reactflow', JSON.stringify({ value, nodeType }));
    event.dataTransfer.effectAllowed = 'move';
  };

  const [value, setValue] = React.useState(0);

  const handleChange = (event: React.SyntheticEvent, newValue: number) => {
    setValue(newValue);
  };

  function a11yProps(index: number) {
    return {
      id: `simple-tab-${index}`,
      'aria-controls': `simple-tabpanel-${index}`,
    };
  }

  const handleListItemClick = (value: AllLogicDescriptions, type: InputNodeType) => {};

  // const generateList = (list: Record<string, AllLogicDescriptions>, type: InputNodeType) => (
  //   <List sx={{ pt: 0 }}>
  //     {Object.keys(list).map((f, i) => (
  //       <ListItemButton onClick={() => handleListItemClick(list[f], type)} key={f + type}>
  //         <ListItemText primary={list[f].name} secondary={list[f].description} />
  //       </ListItemButton>
  //     ))}
  //   </List>
  // );
  const generateList = (list: Record<string, AllLogicDescriptions>, type: InputNodeType) => (
    <List>
      {Object.keys(list).map((f, i) => (
        <ListItem key={JSON.stringify(list[f]) + type + i}>
          <ListItemText
            draggable
            primary={list[f].name}
            secondary={list[f].description}
            onDragStart={(event) => onDragStart(event, list[f], type)}
            key={f + type}
          />
        </ListItem>
      ))}
    </List>
  );

  return (
    <aside className="">
      <Box>
        <Tabs value={value} onChange={handleChange} aria-label="basic tabs example">
          <Tab label="Aggregations" {...a11yProps(0)} />
          <Tab label="Operations" {...a11yProps(1)} />
          <Tab label="Filters" {...a11yProps(2)} />
        </Tabs>
      </Box>
      <CustomTabPanel value={value} index={0}>
        {generateList(MathAggregations, 'float')}
      </CustomTabPanel>
      <CustomTabPanel value={value} index={1}>
        {generateList(MathFunctions, 'float')}
        {generateList(StringFunctions, 'string')}
      </CustomTabPanel>
      <CustomTabPanel value={value} index={2}>
        {generateList(MathFilters, 'float')}
        {generateList(StringFilters, 'string')}
      </CustomTabPanel>
    </aside>
  );
};

export type QueryBuilderProps = {
  onRunQuery?: () => void;
};

export const QueryBuilder = (props: QueryBuilderProps) => {
  return (
    <div
      style={{
        width: '100%',
        height: '22rem',
        display: 'flex',
        gap: '1rem',
      }}
    >
      <QueryBuilderPills />
      <ReactFlowProvider>
        <QueryBuilderInner {...props} />
      </ReactFlowProvider>
    </div>
  );
};

// export const QueryBuilder = () => {
//   const graphologyGraph = useQuerybuilderGraphology();
//   const schema = useSchemaGraphology();
//   const graph = useQuerybuilderGraph();
//   const config = useConfig();
//   const dispatch = useDispatch();

//   return (
//     <QueryBuilderRaw
//       graphologyGraph={graphologyGraph}
//       schema={schema}
//       graph={graph}
//       config={config}
//     />
//   );
// };

export default QueryBuilder;
