/**
 * This program has been developed by students from the bachelor Computer Science at
 * Utrecht University within the Software Project course.
 * © Copyright Utrecht University (Department of Information and Computing Sciences)
 */

/* istanbul ignore file */
/* The comment above was added so the code coverage wouldn't count this file towards code coverage.
 * We do not test components/renderfunctions/styling files.
 * See testing plan for more details.*/
import React, { useState } from 'react';
import { Node, Handle, Position, NodeProps } from 'reactflow';
import styles from './relation.module.scss';
import {
  calcWidthEntityNodeBox,
  calculateAttributeQuality,
  calculateEntityQuality,
  calcWidthRelationNodeBox,
  calculateRelationQuality,
} from '@graphpolaris/shared/lib/schema/schema-utils';
import { useTheme } from '@mui/material';
import { SchemaReactflowRelation, SchemaReactflowRelationWithFunctions } from '../../../model/reactflow';
import { QueryElementTypes } from '@graphpolaris/shared/lib/querybuilder';

/**
 * Relation node component that renders a relation node for the schema.
 * Can be dragged and dropped to the query builder.
 * @param {NodeProps} param0 The data of an entity flow element.
 */
export const RelationNode = React.memo(({ id, data }: NodeProps<SchemaReactflowRelationWithFunctions>) => {
  const [hidden, setHidden] = useState<boolean>(true);
  const theme = useTheme();
  // console.log(data);

  /**
   * Adds drag functionality in order to be able to drag the relationNode to the schema.
   * @param event React Mouse drag event.
   */
  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    console.log('dragging relation', id, data);
    event.dataTransfer.setData(
      'application/reactflow',
      JSON.stringify({
        type: QueryElementTypes.Relation,
        name: id, //TODO id?
        from: data.from,
        to: data.to,
        collection: data.collection,
        label: data.label,
      })
    );
    event.dataTransfer.effectAllowed = 'move';
  };

  /**
   * Displays the NodeQualityPopup when clicked, and hides them when clicked again
   */
  const onClickToggleNodeQualityPopup = (): void => {
    data.toggleNodeQualityPopup(data.collection);
  };

  /**
   * displays the attribute analytics popup menu when clicked, and hides them when clicked again
   */
  const onClickToggleAttributeAnalyticsPopupMenu = (): void => {
    data.toggleAttributeAnalyticsPopupMenu(data.collection);
  };

  const widthExternalBoxes = data.attributes ? calcWidthRelationNodeBox(data.attributes.length, data.nodeCount) : 0;

  return (
    <div
      onDragStart={(event) => onDragStart(event)}
      draggable
      // style={{ width: 100, height: 100 }}
    >
      <div
        className={styles.relationNode}
        style={{
          background: theme.palette.custom.nodesBase[0],
          borderTop: `4px solid ${theme.palette.custom.nodesBase[0]}`,
          borderBottom: `6px solid ${theme.palette.custom.elements.relationBase[0]}`,
        }}
      >
        <div
          className={[styles.relationNodeTriangleGeneral, styles.relationNodeTriangleLeft].join(' ')}
          style={{
            borderRightColor: theme.palette.custom.nodesBase[0],
          }}
        ></div>
        <div
          className={[styles.relationNodeTriangleGeneral, styles.relationNodeSmallTriangleLeft].join(' ')}
          style={{
            borderRightColor: theme.palette.custom.elements.relationBase[0],
          }}
        ></div>
        <div
          className={[styles.relationNodeTriangleGeneral, styles.relationNodeTriangleRight].join(' ')}
          style={{ borderLeftColor: theme.palette.custom.nodesBase[0] }}
        ></div>
        <div
          className={[styles.relationNodeTriangleGeneral, styles.relationNodeSmallTriangleRight].join(' ')}
          style={{
            borderLeftColor: theme.palette.custom.elements.relationBase[0],
          }}
        ></div>
        <Handle
          style={{
            pointerEvents: 'none',
            borderBottomColor: theme.palette.custom.nodesBase[0],
          }}
          className={styles.arrowup}
          id="entitySourceLeft"
          position={Position.Top}
          type="source"
        ></Handle>
        <div
          className={styles.relationNodeAttributesBox}
          onClick={() => onClickToggleAttributeAnalyticsPopupMenu()}
          style={{
            width: widthExternalBoxes + 'px',
            backgroundImage: `linear-gradient(-90deg, 
              ${theme.palette.custom.nodesBase} 0%, 
              ${theme.palette.custom.nodesBase} ${calculateAttributeQuality(data)}%, 
              ${theme.palette.custom.elements.relationSecond[0]} ${calculateAttributeQuality(data)}%)`,
          }}
        >
          <span
            style={{
              paddingLeft: '25px',
              float: 'left',
            }}
          >
            A
          </span>
          <span className={styles.nodeSpan}>{data.attributes.length}</span>
        </div>
        <div
          className={styles.relationNodeNodesBox}
          onClick={() => onClickToggleNodeQualityPopup()}
          style={{
            width: widthExternalBoxes + 'px',
            backgroundImage: `linear-gradient(-90deg, 
              ${theme.palette.custom.nodesBase} 0%, 
              ${theme.palette.custom.nodesBase} ${calculateRelationQuality(data)}%, 
              ${theme.palette.custom.elements.relationSecond[0]} ${calculateRelationQuality(data)}%)`,
          }}
        >
          <span
            style={{
              paddingLeft: '25px',
              float: 'left',
            }}
          >
            N
          </span>
          <span className={styles.nodeSpan}>{data.nodeCount}</span>
        </div>

        <div className={styles.nodeWrapper} style={{ backgroundColor: theme.palette.custom.nodesBase[0] }}>
          <span className={styles.nodeData}>{data.collection}</span>
        </div>

        <Handle
          className={styles.arrowdown}
          style={{
            pointerEvents: 'none',
            borderTopColor: theme.palette.custom.elements.relationBase[0],
          }}
          position={Position.Bottom}
          type="target"
        ></Handle>
      </div>
    </div>
  );
});

RelationNode.displayName = 'RelationNode';
